/* 
   Hud.cpp

   SpaceShipChase 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#include "Hud.h"

Hud::Hud()
{
}

Hud::~Hud()
{
}

/**
 * switch to 2d drawing mode
 */
void Hud::start2DMode()
{
  glPushAttrib(GL_ENABLE_BIT);
  
  glMatrixMode(GL_PROJECTION);                        
  
  glPushMatrix();                                    
  glLoadIdentity();                                
  
  glOrtho(0, 1024, 0, 768, -100, 100);    
  
  // Switch to model view so that we can render the scope image
  glMatrixMode(GL_MODELVIEW);                                
  glPushMatrix();
  // Initialize the current model view matrix with the identity matrix
  glLoadIdentity(); 
}

/**
 * switch back to 3d mode
 */
void Hud::end2DMode()
{
  glDisable(GL_BLEND);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_CULL_FACE);
  
  glMatrixMode(GL_PROJECTION); 
  glPopMatrix();                                            
  
  // Go back to our model view matrix 
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();

  glPopAttrib();
}

/**
 * convert an SDL surface to a GL texture
 */
GLuint Hud::GLLoadTexture(SDL_Surface *surface, GLfloat *texcoord)
{
  GLuint texture;
  int w, h;
  SDL_Surface *image;
  SDL_Rect area;
  Uint32 savedFlags;
  Uint8 savedAlpha;

  w = powerOfTwo(surface->w);
  h = powerOfTwo(surface->h);
  texcoord[0] = 0.0f;
  texcoord[1] = 0.0f;
  texcoord[2] = (GLfloat)surface->w / (GLfloat)w;
  texcoord[3] = (GLfloat)surface->h / (GLfloat)h;

  image = SDL_CreateRGBSurface(SDL_SWSURFACE, 
			       w, h, 
			       32,
#if SDL_BYTEORDER == SDL_LIL_ENDIAN // opengl RGBA masks
			       0x000000FF, 
			       0x0000FF00, 
			       0x00FF0000, 
			       0xFF000000
#else
			       0xFF000000,
			       0x00FF0000, 
			       0x0000FF00, 
			       0x000000FF
#endif
			       );
  
  if (image == NULL) // make sure we created a valid surface
    return 0;

  savedFlags = surface->flags&(SDL_SRCALPHA | SDL_RLEACCELOK);
  savedAlpha = surface->format->alpha;
  if ((savedFlags & SDL_SRCALPHA) == SDL_SRCALPHA)
    SDL_SetAlpha(surface, 0, 0);

  // copy the surface into a gl texture
  area.x = 0;
  area.y = 0;
  area.w = surface->w;
  area.h = surface->h;
  SDL_BlitSurface(surface, &area, image, &area);

  // restore the alpha attributes
  if ((savedFlags & SDL_SRCALPHA) == SDL_SRCALPHA)
    SDL_SetAlpha(surface, savedFlags, savedAlpha);

  // create the opengl texture
  glGenTextures(1, &texture);
  glBindTexture(GL_TEXTURE_2D, texture);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, image->pixels);

  // free the surface
  SDL_FreeSurface(image);

  return texture;
}

/**
 * helper to make the texture a power of 2
 */
int Hud::powerOfTwo(int input)
{
  int value = 1;
  
  while ( value < input ) 
    value <<= 1;
   
  return value;
}
