/* 
   HudMessage.cpp

   SpaceShipChase 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#include "HudMessage.h"

HudMessage::HudMessage()
  : Hud()
{
  _font = 0;
  _font = TTF_OpenFont("data/WAR2.TTF", 56);
  if (!_font)
    cerr << "Error loading font\n";

  _timerLength = 0.0;
  _timer.reset();
}

HudMessage::~HudMessage()
{
  if (_font)
    TTF_CloseFont(_font);
}

HudMessage *HudMessage::getInstance()
{
  static HudMessage a;
  return &a;
}

/**
 * draw the hud
 */
void HudMessage::draw()
{
  if (_timer.update() >= _timerLength) // passed time period
    return;

  // draw
  glColor3f(1.0, 1.0, 1.0);

  start2DMode();

  glDisable(GL_CULL_FACE);
  glDisable(GL_DEPTH_TEST);
  
  // Select our desired depth testing and turn on blending
  glBlendFunc(GL_DST_COLOR, GL_ZERO);
  glEnable(GL_BLEND);
  glEnable(GL_TEXTURE_2D);
 
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glBindTexture(GL_TEXTURE_2D, _messageTexture[0]);
  glBegin(GL_TRIANGLE_STRIP);
    glTexCoord2f(_messageCoordinates[0], _messageCoordinates[1]);    
    glVertex3f(_messageLeftPosition, _messageTopPosition + _messageH, 0.0);
    glTexCoord2f(_messageCoordinates[2], _messageCoordinates[1]);
    glVertex3f(_messageLeftPosition + _messageW, _messageTopPosition + _messageH, 0.0);
    glTexCoord2f(_messageCoordinates[0], _messageCoordinates[3]);
    glVertex3f(_messageLeftPosition, _messageTopPosition, 0.0);
    glTexCoord2f(_messageCoordinates[2], _messageCoordinates[3]);
    glVertex3f(_messageLeftPosition + _messageW, _messageTopPosition, 0.0);
  glEnd();
  
  glBlendFunc(GL_SRC_ALPHA,GL_ONE);

  end2DMode();
}

/**
 * add a new message
 */
void HudMessage::addMessage(string message, double time)
{
  if (_messageTexture[0] < 1) // delete
    glDeleteTextures(1, &_messageTexture[0]);

  SDL_Surface *text = 0;
  SDL_Color green = { 0x00, 0xFF, 0x00, 0 };

  _timerLength = time;

  if (_font)
    text = TTF_RenderText_Blended(_font, message.c_str(), green);

  if (!text) // error creating surfrace
  {
    cerr << "Bad surface\n";
    return;
  }

  _messageW = text->w;
  _messageH = text->h;

  SystemConfig *config = SystemConfig::getInstance();
  _messageTopPosition = (config->getHeight() / 2) + (_messageH / 2);
  _messageLeftPosition = (config->getWidth() / 2) - (_messageW / 2);

  _messageTexture[0] = GLLoadTexture(text, _messageCoordinates);

  // free the surface
  SDL_FreeSurface(text);

  _timer.reset();
}
