/* 
   MainMenu.h

   SpaceShipChase 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#ifndef _MAINMENU_H_
#define _MAINMENU_H_

#include <iostream>
using namespace std;

#include "Camera.h"
#include "Hud.h"
#include "SystemConfig.h"
#include "Timer.h"

/**
 * the main menu for the game
 */
class MainMenu : public Hud
{
 public:
  static MainMenu *getInstance();
  /**
   * draw the hud
   */
  virtual void draw();
  /**
   * run the main menu program
   */
  virtual bool run();

 protected:
  /**
   * load the menu
   */
  void loadMenu();
  /**
   * check for input
   */
  bool handleKeyPress();

  // font
  TTF_Font *_font;

  // width and height of the play in the top right
  int _playW, _playH;
  // coordinates for the play
  GLfloat _playCoordinates[4];
  // play texture
  GLuint _playTexture[1];
  // top left corner of play position
  double _playTopPosition, _playLeftPosition;

  // width and height of the quit in the top right
  int _quitW, _quitH;
  // coordinates for the quit
  GLfloat _quitCoordinates[4];
  // quit texture
  GLuint _quitTexture[1];
  // top left corner of quit position
  double _quitTopPosition, _quitLeftPosition;

  // background
  GLuint _backgroundTexture[1];

  string _current;
  GLfloat _rotateY;
  bool _switchDirection;

 private:
  MainMenu();
  virtual ~MainMenu();
};

#endif
