/* 
   SpaceShip.cpp

   SpaceShipChase 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#include "SpaceShip.h"
#include "World.h"
#include "Actor.h"

SpaceShip::SpaceShip()
  : Actor()
{
  /*
  vector < string > animationNames;
  vector < int> animations;

  // still
  animationNames.push_back("mesh/Duke_Still.jba");
  animations.push_back(STILL);
  // walk
  animationNames.push_back("mesh/Duke_Walk.jba");
  animations.push_back(WALK);

  loadMesh("mesh/Duke.jbm", animationNames, animations);
  */
  _pauseTimer.reset();
  _invincibleTimer.reset();
  _collisionTimer.reset();

  loadMesh("mesh/SpaceShip.jbm");  
  
  _collisionRadius = 1.0;
  
  _drawBeam = false;
  Texture *t = Texture::getInstance();
  _beamTexture = t->createTexture("images/Beam.tga");
  
  _lastDistance = 10.0;
  _speedFactor = 0.6;
  
  _aggressiveness = 1;

  _spinDegrees = 0.0;

  _beamTextureTranslate = 0.0;

  _startedCrash = false;
}

SpaceShip::~SpaceShip()
{
}

/**
 * draw
 */
void SpaceShip::draw()
{
  glPushMatrix();

  glTranslatef(_currentTranslate.getX() + _initialTranslate.getX(),
	       _currentTranslate.getY() + _initialTranslate.getY(),
	       _currentTranslate.getZ() + _initialTranslate.getZ());
  glRotatef(_currentRotate.getX(), 1, 0, 0);
  //glRotatef(_currentRotate.getY(), 0, 1, 0);
  glRotatef(_spinDegrees, 0, 1, 0);
  glRotatef(_currentRotate.getZ(), 0, 0, 1);
  glScalef(_initialScale.getX(), _initialScale.getY(), _initialScale.getZ());

  if (_mesh)
    _mesh->draw();

  glPopMatrix();
  
  if (_drawBeam)
  {
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);

    // translate the texture
    glMatrixMode(GL_TEXTURE);
    glLoadIdentity();
    glTranslatef(0.0, _beamTextureTranslate, 0.0);
    glBindTexture(GL_TEXTURE_2D, _beamTexture);
    glMatrixMode(GL_MODELVIEW);

    glPushMatrix();
    glTranslatef(_currentTranslate.getX() + _initialTranslate.getX(),
		 0.0,
		 _currentTranslate.getZ() + _initialTranslate.getZ());

  
    glDisable(GL_CULL_FACE);
    // right side
    glBegin(GL_QUADS);
    glTexCoord2f(0.0, 0.0);
    glVertex3f(1.0, -0.5, 1.0);
    glTexCoord2f(1.0, 0.0);
    glVertex3f(1.0, -0.5, -1.0);
    glTexCoord2f(1.0, 1.0);
    glVertex3f(0.5, 4.0, -0.5);
    glTexCoord2f(0.0, 1.0);
    glVertex3f(0.5, 4.0, 0.5);
    glEnd();

    // left side
    glBegin(GL_QUADS);
    glTexCoord2f(0.0, 0.0);
    glVertex3f(-1.0, -0.5, -1.0);
    glTexCoord2f(1.0, 0.0);
    glVertex3f(-1.0, -0.5, 1.0);
    glTexCoord2f(1.0, 1.0);
    glVertex3f(-0.5, 4.0, 0.5);
    glTexCoord2f(0.0, 1.0);
    glVertex3f(-0.5, 4.0, -0.5);
    glEnd();

    // front
    glBegin(GL_QUADS);
    glTexCoord2f(0.0, 0.0);
    glVertex3f(-1.0, -0.5, 1.0);
    glTexCoord2f(1.0, 0.0);
    glVertex3f(1.0, -0.5, 1.0);
    glTexCoord2f(1.0, 1.0);
    glVertex3f(0.5, 4.0, 0.5);
    glTexCoord2f(0.0, 1.0);
    glVertex3f(-0.5, 4.0, 0.5);
    glEnd();

    glEnable(GL_CULL_FACE);
    
    glPopMatrix();
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_BLEND);

    glMatrixMode(GL_TEXTURE);
    glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);

    // update the beam texture translate
    Camera *c = Camera::getInstance();
    _beamTextureTranslate -= c->getCurrentFPS();
  }
}

/**
 * pause the ship momentarily
 */
void SpaceShip::pause()
{
  if (_invincibleTimer.update() > 1.1)
  {
    _pauseTimer.reset();
    _invincibleTimer.reset();
  }
}

/**
 * update the character
 *  and either handle input
 *  or run AI code
 */
void SpaceShip::update()
{
  if (_pauseTimer.update() > 1.0)
  {
    _drawBeam = false;
    if (_ai)
      runAI();
    else
      handleInput();
  }

  Camera *c = Camera::getInstance();
  // rotate
  _spinDegrees += c->getCurrentFPS() * 100;
  if (_spinDegrees > 360)
    _spinDegrees -= _spinDegrees;
}

/**
 * handle keyboard/joystick input
 */
void SpaceShip::handleInput()
{
    Actor::handleInput();
    
   // check for space for beam
    Uint8 *keyState = SDL_GetKeyState (NULL); 
    if (keyState[SDLK_SPACE])
      fireBeam();
}

/**
 * fire the beam to capture the
 *  single player
 */
void SpaceShip::fireBeam()
{
  // pause us
  pause();

  World *w;
  w = World::getInstance();
  
  Actor *mainPlayer = w->getPlayer1();
  
  Vector3 myTranslate = getAllTranslate();
  Vector3 p1Translate = mainPlayer->getAllTranslate();
  
  if ((fabs(myTranslate.getX() - p1Translate.getX()) < 0.3) &&
      (fabs(myTranslate.getZ() - p1Translate.getZ()) < 0.3))
  {
    w->setCaptured(true);
  }
  
  _drawBeam = true;
}

/**
 * run the ai code
 */
void SpaceShip::runAI()
{
  // get the other two ships and the player
  Actor *ship1, *ship2;
  Actor *player;
  // get the world
  World *w = World::getInstance();
  
  player = w->getPlayer1();
  
  if (w->getShip1()->getID() == getID())
  {
    ship1 = w->getShip2();
    ship2 = w->getShip3();
  }
  else if (w->getShip2()->getID() == getID())
  {
    ship1 = w->getShip1();
    ship2 = w->getShip3();
  }
  else
  {
    ship1 = w->getShip1();
    ship2 = w->getShip2();
  }
  
  // start chasing the player
  Vector3 playerPosition = player->getAllTranslate();
  Vector3 myPosition = getAllTranslate();
  
  double currentDistance = sqrt(pow(playerPosition.getX() - myPosition.getX(), 2) + pow(playerPosition.getZ() - myPosition.getZ(), 2));
  
  // check if we should fire the beam or move
  if (fabs(playerPosition.getX() - myPosition.getX()) < 0.1 &&
      fabs(playerPosition.getZ() - myPosition.getZ()) < 0.1)
  {
    fireBeam();
  }
  else
  {
    if (fabs(playerPosition.getX() - myPosition.getX()) < 0.9 &&
      fabs(playerPosition.getZ() - myPosition.getZ()) < 0.9)
    {
      if (_lastDistance < currentDistance && fabs(_lastDistance - currentDistance) < 0.9)
        fireBeam();
    }
    
    double half = 45.0 / 2.0;
    double angle = atan2(playerPosition.getZ() - myPosition.getZ(), playerPosition.getX() - myPosition.getX());
    // convert to degrees
    angle = angle * (180 / M_PI);
  
    if (_aggressiveness == 1)
    {
      if (angle >= -half && angle <= half) // east
        _currentDirection = RIGHT;
      else if (angle >= 45 - half && angle <= 45 + half) // southeast
        _currentDirection = DOWNRIGHT;
      else if (angle >= 90 - half && angle <= 90 + half) // south
        _currentDirection =  DOWN;
      else if (angle >= 135 - half && angle <= 135 + half) // sourh west
        _currentDirection =  DOWNLEFT;
      else if (angle >= 180 - half || angle <= -180 + half) // west
        _currentDirection =  LEFT;
      else if (angle >= -135 - half && angle <= -135 + half) // north west
        _currentDirection = UPLEFT;
      else if (angle >= -90 - half && angle <= -90 + half) // north
        _currentDirection = UP;
      else //north east
        _currentDirection = UPRIGHT;

      // make sure we are way away from the other 2 ships
      int playerDirection = -1;
      if (sqrt(pow(ship1->getAllTranslate().getX() - myPosition.getX(), 2) + pow(ship1->getAllTranslate().getZ() - myPosition.getZ(), 2)) < 2.5)
	playerDirection = dynamic_cast < SpaceShip * > (ship1)->getCurrentDirection();
      if (sqrt(pow(ship2->getAllTranslate().getX() - myPosition.getX(), 2) + pow(ship2->getAllTranslate().getZ() - myPosition.getZ(), 2)) < 2.5)
	playerDirection = dynamic_cast < SpaceShip * > (ship2)->getCurrentDirection();
      
      if (playerDirection != -1)
      {
	switch (playerDirection)
	{
	case UP:
	  _currentDirection = UPRIGHT;
	  break;
	case DOWN:
	  _currentDirection = DOWNRIGHT;
	  break;
	case LEFT:
	  _currentDirection = UPLEFT;
	  break;
	case RIGHT:
	  _currentDirection = UPRIGHT;
	case DOWNRIGHT:
	  _currentDirection = DOWN;
	  break;
	case DOWNLEFT:
	  _currentDirection = LEFT;
	  break;
	case UPRIGHT:
	  _currentDirection = RIGHT;
	  break;
	case UPLEFT:
	  _currentDirection = UP;
	  break;
	}
      }

      move(_currentDirection);
    }
    else if (_aggressiveness == 2)
    {
      if (angle >= -45 && angle <= 45) // east
        _currentDirection = RIGHT;
      else if (angle >= 45 && angle <= 135) // south
        _currentDirection = DOWN;
      else if (angle >= 135 || angle <= -135) // west
        _currentDirection = LEFT;
      else if (angle >= -135 && angle <= -45) // north
        _currentDirection = UP;
    
      if (_collisionTimer.update() < 1.4) // pause time + extra
      {
        switch(_currentDirection)
        {
          case RIGHT:
            move(LEFT);
          case LEFT:
            move(RIGHT);
          case UP:
            move(DOWN);
          case DOWN:
            move(UP);
        }
      }
      else
      {
        move(_currentDirection);
      }
    }
    else if (_aggressiveness == 3)
    {
      if (angle >= -half && angle <= half) // east
        move(RIGHT);
      else if (angle >= 45 - half && angle <= 45 + half) // southeast
        move(DOWNRIGHT);
      else if (angle >= 90 - half && angle <= 90 + half) // south
        move (DOWN);
      else if (angle >= 135 - half && angle <= 135 + half) // sourh west
        move (DOWNLEFT);
      else if (angle >= 180 - half || angle <= -180 + half) // west
        move (LEFT);
      else if (angle >= -135 - half && angle <= -135 + half) // north west
        move(UPLEFT);
      else if (angle >= -90 - half && angle <= -90 + half) // north
        move(UP);
      else //north east
        move(UPRIGHT);
    }
  }
  
  _lastDistance = currentDistance;
}

/**
 * set the aggressiveness level
 */
void SpaceShip::setAggressiveness(int i)
{
  _aggressiveness = i;
  
  if (i == 1)
    _speedFactor = 0.8;
  else if (i == 2)
    _speedFactor = 0.4;
  else
    _speedFactor = 0.6;
}

/**
 * notify if collided
 */
void SpaceShip::shipCollision()
{
  if (_collisionTimer.update() > 1.3)
    _collisionTimer.reset();
}

/**
 * do the ending crash sequence
 */
void SpaceShip::crash()
{
  if (!_startedCrash)
  {
    _crashTimer.reset();
    _startedCrash = true;;
  }
  double time = _crashTimer.update();

  if (time < 1.5)
  {
    Vector3 r = getRotate();
    
    double x = time * 100;
    if (x > 60)
      x = (int)x % 60;
    if (x > 15)
      x = 15 - (x - 15);
    if (x < -15)
      x = -15 + (-15 - x);
    
    double z = time * 50;
    if (z > 60)
      z = (int)z % 60;
    if (z > 15)
      z = 15 - (z - 15);
    if (z < -15)
      z = -15 + (-15 - z);
    
    r.setX(x);
    r.setZ(z);
    setRotate(r);
    
    Vector3 v = getTranslate();
    v.setY(-5.3 * (.33333 * (time * 1.5)));
    setTranslate(v);
  }
}
