/* 
   Actor.h

   SpaceShipChase 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#ifndef _ACTOR_H_
#define _ACTOR_H_

#include "Object.h"

// enum for directions
enum
  {
    UP = 0,
    DOWN,
    LEFT,
    RIGHT,
    UPRIGHT,
    UPLEFT,
    DOWNRIGHT,
    DOWNLEFT
  };

class Actor : public Object
{
 public:
  Actor();
  virtual ~Actor();

  /**                                                                                                                                                                                          
   * update the character                                                                                                                                         
   *  and either handle input 
   *  or run AI code                                                                                                                                                                                          
   */
  virtual void update();
  /**
   * handle the input
   */
  virtual void handleInput();

  /**
   * set the keys, up down left and right
   */
  virtual void setKeys(int up, int down, int left, int right);
  /**
   * sets the joystick number
   */
  virtual void setJoystick(SDL_Joystick *i) { _joystick = i; }
  /**
   * is this actor ai ?
   */
  void setAI(bool t) { _ai = t; }

 protected:
  /**
   * move in the specific direction
   */
  virtual void move(int direction);
  /**
   * run the ai code
   */
  virtual void runAI();

  // for keypresses
  int _keyUp, _keyDown, _keyLeft, _keyRight;
  // for joysticks
  SDL_Joystick *_joystick;
  // ai or not?
  bool _ai;
  // how fast to move the actor
  double _speedFactor;
};

#endif
