/* 
   HudInGame.cpp

   SpaceShipChase 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#include "HudInGame.h"

HudInGame::HudInGame()
  : Hud()
{
  _font = 0;
  _font = TTF_OpenFont("data/WAR2.TTF", 28);
  if (!_font)
    cerr << "Error loading font\n";

  _counterTopPosition = 768 - 45;
  _counterLeftPosition = 1024 - 65;

  update(30.0);
}

HudInGame::~HudInGame()
{
  if (_font)
    TTF_CloseFont(_font);
}

HudInGame *HudInGame::getInstance()
{
  static HudInGame a;

  return &a;
}

/**
 * draw the hud
 */
void HudInGame::draw()
{
  if (_counterTexture[0] < 1)
    return;

  glColor3f(1.0, 1.0, 1.0);

  start2DMode();

  glDisable(GL_CULL_FACE);
  glDisable(GL_DEPTH_TEST);
  
  // Select our desired depth testing and turn on blending
  glBlendFunc(GL_DST_COLOR, GL_ZERO);
  glEnable(GL_BLEND);
  glEnable(GL_TEXTURE_2D);
 
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glBindTexture(GL_TEXTURE_2D, _counterTexture[0]);
  glBegin(GL_TRIANGLE_STRIP);
    glTexCoord2f(_counterCoordinates[0], _counterCoordinates[1]);    
    glVertex3f(_counterLeftPosition, _counterTopPosition + _counterH, 0.0);
    glTexCoord2f(_counterCoordinates[2], _counterCoordinates[1]);
    glVertex3f(_counterLeftPosition + _counterW, _counterTopPosition + _counterH, 0.0);
    glTexCoord2f(_counterCoordinates[0], _counterCoordinates[3]);
    glVertex3f(_counterLeftPosition, _counterTopPosition, 0.0);
    glTexCoord2f(_counterCoordinates[2], _counterCoordinates[3]);
    glVertex3f(_counterLeftPosition + _counterW, _counterTopPosition, 0.0);
  glEnd();
  
  glBlendFunc(GL_SRC_ALPHA,GL_ONE);

  end2DMode();
}

/**
 * update the time
 */
void HudInGame::update(float time)
{
  if (_counterTexture[0] < 1) // delete
    glDeleteTextures(1, &_counterTexture[0]);

  SDL_Surface *text = 0;
  SDL_Color green = { 0x00, 0xFF, 0x00, 0 };

  ostringstream m;
  m << (int)time;
  if (_font)
    text = TTF_RenderText_Blended(_font, m.str().c_str(), green);

  if (!text) // error creating surfrace
  {
    cerr << "Bad surface\n";
    return;
  }

  _counterW = text->w;
  _counterH = text->h;

  _counterTexture[0] = GLLoadTexture(text, _counterCoordinates);

  // free the surface
  SDL_FreeSurface(text);
}
