/* 
   MainMenu.cpp

   SpaceShipChase 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#include "MainMenu.h"

MainMenu::MainMenu()
  : Hud()
{
  _font = 0;
  _font = TTF_OpenFont("data/WAR2.TTF", 56);
  if (!_font)
    cerr << "Error loading font\n";

  _current = "play";
  _rotateY = 0.0;
  _switchDirection = false;

  loadMenu();
}

MainMenu::~MainMenu()
{
  if (_font)
    TTF_CloseFont(_font);
}

MainMenu *MainMenu::getInstance()
{
  static MainMenu a;
  return &a;
}

/**
 * draw the hud
 */
void MainMenu::draw()
{
  // draw
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();
 
  Camera *c = Camera::getInstance();
  c->update();
  c->look();

  SystemConfig *config = SystemConfig::getInstance();

  glColor3f(1.0, 1.0, 1.0);

  start2DMode();

  glEnable(GL_TEXTURE_2D);

  // background
  glBindTexture(GL_TEXTURE_2D, _backgroundTexture[0]);
  glBegin(GL_QUADS);
    glTexCoord2f(0.0, 0.0);
    glVertex3f(0.0, 0.0, 0.0);
    glTexCoord2f(1.0, 0.0);
    glVertex3f(config->getWidth(), 0.0, 0.0);
    glTexCoord2f(1.0, 1.0);
    glVertex3f(config->getWidth(), config->getHeight(), 0.0);
    glTexCoord2f(0.0, 1.0);
    glVertex3f(0.0, config->getHeight(), 0.0);
  glEnd();

  glDisable(GL_CULL_FACE);
  glDisable(GL_DEPTH_TEST);
  
  // Select our desired depth testing and turn on blending
  glBlendFunc(GL_DST_COLOR, GL_ZERO);
  glEnable(GL_BLEND);

  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glBindTexture(GL_TEXTURE_2D, _playTexture[0]);
  glPushMatrix();
  if (_current == "play")
  {
    glTranslatef((_playLeftPosition + (_playW / 2)), (_playTopPosition + (_playH / 2)), 0.0);
    glRotatef(_rotateY, 0, 0, 1);
    glTranslatef(-(_playLeftPosition + (_playW / 2)), -(_playTopPosition + (_playH / 2)), 0.0);
  }
  glBegin(GL_TRIANGLE_STRIP);
    glTexCoord2f(_playCoordinates[0], _playCoordinates[1]);    
    glVertex3f(_playLeftPosition, _playTopPosition + _playH, 0.0);
    glTexCoord2f(_playCoordinates[2], _playCoordinates[1]);
    glVertex3f(_playLeftPosition + _playW, _playTopPosition + _playH, 0.0);
    glTexCoord2f(_playCoordinates[0], _playCoordinates[3]);
    glVertex3f(_playLeftPosition, _playTopPosition, 0.0);
    glTexCoord2f(_playCoordinates[2], _playCoordinates[3]);
    glVertex3f(_playLeftPosition + _playW, _playTopPosition, 0.0);
  glEnd();
  glPopMatrix();

  glBindTexture(GL_TEXTURE_2D, _quitTexture[0]);
  glPushMatrix();
  if (_current == "quit")
  {
    glTranslatef((_quitLeftPosition + (_quitW / 2)), (_quitTopPosition + (_quitH / 2)), 0.0);
    glRotatef(_rotateY, 0, 0, 1);
    glTranslatef(-(_quitLeftPosition + (_quitW / 2)), -(_quitTopPosition + (_quitH / 2)), 0.0);
  }
  glBegin(GL_TRIANGLE_STRIP);
    glTexCoord2f(_quitCoordinates[0], _quitCoordinates[1]);    
    glVertex3f(_quitLeftPosition, _quitTopPosition + _quitH, 0.0);
    glTexCoord2f(_quitCoordinates[2], _quitCoordinates[1]);
    glVertex3f(_quitLeftPosition + _quitW, _quitTopPosition + _quitH, 0.0);
    glTexCoord2f(_quitCoordinates[0], _quitCoordinates[3]);
    glVertex3f(_quitLeftPosition, _quitTopPosition, 0.0);
    glTexCoord2f(_quitCoordinates[2], _quitCoordinates[3]);
    glVertex3f(_quitLeftPosition + _quitW, _quitTopPosition, 0.0);
  glEnd();
  glPopMatrix();
  
  glBlendFunc(GL_SRC_ALPHA,GL_ONE);

  end2DMode();

  SDL_GL_SwapBuffers();

  if (_rotateY > 10.0)
    _switchDirection = true;
  if (_rotateY < -10.0)
    _switchDirection = false;

  if (_switchDirection)
    _rotateY -= c->getCurrentFPS() * 40;
  else
    _rotateY += c->getCurrentFPS() * 40;
}

/**
 * run the main menu program
 */
bool MainMenu::run()
{
  while (1)
  {
    draw();
    bool keyPress = handleKeyPress();
    if (keyPress)
    {
      if (_current == "play")
	return true;
      else
	return false;
    }
  }
  return false;
}

/**
 * load the menu
 */
void MainMenu::loadMenu()
{
  // create the background texture
  Texture *t = Texture::getInstance();
  _backgroundTexture[0] = t->createTexture("images/MainMenuBackground.png");

  SDL_Surface *text = 0;
  SDL_Color green = { 0x00, 0xFF, 0x00, 0 };

  // load the play
  if (_font)
    text = TTF_RenderText_Blended(_font, "Play", green);

  if (!text) // error creating surfrace
  {
    cerr << "Bad surface\n";
    return;
  }
  _playW = text->w;
  _playH = text->h;

  SystemConfig *config = SystemConfig::getInstance();
  _playTopPosition = (config->getHeight() / 2) + (_playH);
  _playLeftPosition = ((double)config->getWidth() / (double)(1.5)) - (_playW / 2);

  _playTexture[0] = GLLoadTexture(text, _playCoordinates);

  SDL_FreeSurface(text);
  text = 0;

  // load the quit
    if (_font)
    text = TTF_RenderText_Blended(_font, "Quit", green);

  if (!text) // error creating surfrace
  {
    cerr << "Bad surface\n";
    return;
  }
  _quitW = text->w;
  _quitH = text->h;

  _quitTopPosition = (config->getHeight() / 2) - (_quitH / 2);
  _quitLeftPosition = ((double)config->getWidth() / (double)(1.5)) - (_quitW / 2);

  _quitTexture[0] = GLLoadTexture(text, _quitCoordinates);

  SDL_FreeSurface(text);
  text = 0;
}

/**
 * check for input
 */
bool MainMenu::handleKeyPress()
{
  SDL_Event event;
  while (SDL_PollEvent(&event))
  {
    switch (event.type)
    {
    case SDL_KEYDOWN:
      switch (event.key.keysym.sym)
      {
      case SDLK_UP:
	if (_current == "quit")
	  _current = "play";
	break;
      case SDLK_DOWN:
	if (_current == "play")
	  _current = "quit";
	break;
      case SDLK_RETURN:
	return true;
	break;
      default:
	break;
      }
      break;
    default:
      break;
    }
  }

  return false;
}
