/* 
   World.h

   SpaceShipChase 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#ifndef _WORLD_H_
#define _WORLD_H_

#include "Actor.h"
#include "Duke.h"
#include "HudInGame.h"
#include "Object.h"
#include "SpaceShip.h"
#include "Timer.h"

/**
 * Holds all the objects
 *  in the world
 *    ****
 *  Singleton
 */
class World : public Object
{
 public:
  /**
   * get an instance of the world
   */
  static World *getInstance();

  /**
   * draw the world
   *  and all its objects
   */
  virtual void draw();
  /**
   * update the world
   *  and all its objects
   *
   * returns 
   * -1 if the game is still going on,
   * 0 if draw
   * 1 if player one won
   * 2 if player two won
   */
  virtual int update();
  /**
   * reset the world to play again
   */
  virtual void reset();
  /**
   * get player 1
   */
  Actor *getPlayer1() { return _player1; }
  SpaceShip *getShip1() { return _ship1; }
  SpaceShip *getShip2() { return _ship2; }
  SpaceShip *getShip3() { return _ship3; }

  /**
   * let the world know if the player is captured
   */
  void setCaptured(bool t) { _captured = t; }
  
  /**
   * do the final dancing animation
   *  1=player1 wins
   *  2=spaceships win
   */
  void doFinalAnimation(int winner);

 protected:
  /**
   * initialize everything
   */
  void init();
  /**
   * deinitialize everything
   */
  void deinit();

  /**
   * the main players
   */
  Duke *_player1;
  /**
   * the space ships
   */
  SpaceShip *_ship1, *_ship2, *_ship3;

  /**
   * check to see if a player
   *  caught an ice cream or
   *  if it hit the floor
   */
  void checkCollision();
  /**
   * check for keyboard/joystick
   *  input for moving the player
   */
  void updatePlayersMovement();

  /**
   * a timer to see when to quit
   */
  Timer _gameTimer;
  Timer _updateTimer;

  /**
   * has the player been captured
   */
  bool _captured;

  /**
   * for the background
   */
  GLuint _spaceTexture[1];

 private:
  World();
  virtual ~World();
};

#endif
