/* 
   Duke.cpp

   SpaceShipChase 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#include "Duke.h"

#include "World.h"

Duke::Duke()
  : Actor()
{
  vector < string > animationNames;
  vector < int> animations;

  // still
  animationNames.push_back("mesh/Duke_Still.jba");
  animations.push_back(STILL);
  // walk
  animationNames.push_back("mesh/Duke_Walk.jba");
  animations.push_back(WALK);

  loadMesh("mesh/Duke.jbm", animationNames, animations);

  _collisionRadius = 1.0;

  _speedFactor = 1.3;
}

Duke::~Duke()
{
}

/**
 * draw the actor and it's code
 */
void Duke::draw()
{
  glPushMatrix();
  
  glTranslatef(_currentTranslate.getX() + _initialTranslate.getX(),
	       _currentTranslate.getY() + _initialTranslate.getY(),
	       _currentTranslate.getZ() + _initialTranslate.getZ());
  glRotatef(_currentRotate.getX(), 1, 0, 0);
  glRotatef(_currentRotate.getY(), 0, 1, 0);
  glRotatef(_currentRotate.getZ(), 0, 0, 1);

  // scale up the character a little
  glPushMatrix();
  glScalef(1.5, 1.5, 1.5);
  _mesh->draw();
  glPopMatrix(); // pop scale

  glPopMatrix(); // pop all
}

/**
 * handle the input
 */
void Duke::handleInput()
{
  Uint8 *keyState = SDL_GetKeyState (NULL);
  double radians = 0.0;
  Camera *camera = Camera::getInstance();
 
  double speed = (camera->getSpeed() / 3.0) * camera->getCurrentFPS();

  Vector3 rotate = getRotate();

  int direction = -1;

  if (_joystick)
  {
    int x, y;
    x = SDL_JoystickGetAxis(_joystick, 0);
    y = SDL_JoystickGetAxis(_joystick, 1);

    if (x > 0)
    {
      if (y < 0)
	direction = UPRIGHT;
      else if (y > 0)
	direction = DOWNRIGHT;
      else
	direction = RIGHT;
    }
    else if (x < 0)
    {
      if (y < 0)
	direction = UPLEFT;
      else if (y > 0)
	direction = DOWNLEFT;
      else
	direction = LEFT;
    }
    else
    {
      if (y < 0)
	direction = UP;
      else if (y > 0)
	direction = DOWN;
    }
  }
  else
  {
    if (keyState[_keyUp] && keyState[_keyLeft]) // move up and left at 45
      direction = UPLEFT;
    else if (keyState[_keyUp] && keyState[_keyRight]) // move up and right at 45
      direction = UPRIGHT;
    else if (keyState[_keyUp]) // move up
      direction = UP;
    else if (keyState[_keyDown] && keyState[_keyLeft]) // down left at 45
      direction = DOWNLEFT;
    else if (keyState[_keyDown] && keyState[_keyRight]) // down right at 45
      direction = DOWNRIGHT;
    else if (keyState[_keyDown]) // down
      direction = DOWN;
    else if (keyState[_keyLeft]) // just left
      direction = LEFT;
    else if (keyState[_keyRight]) // just right
      direction = RIGHT;
  }

  move(direction);
}

/**
 * run the ai code
 */
void Duke::runAI()
{

}
