/* 
   SpaceShip.h

   SpaceShipChase 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#ifndef _SPACESHIP_H_
#define _SPACESHIP_H_

#include "Actor.h"
#include "Timer.h"

/**
 * class for the space ship character
 */
class SpaceShip : public Actor
{
 public:
  SpaceShip();
  virtual ~SpaceShip();

  /**
   * update the character 
   *  and either handle input
   *  or run AI code
   */
  virtual void update();
  /**
   * draw the actor and it's code
   */
  virtual void draw();
  /**
   * handle the input
   */
  virtual void handleInput();
  /**
   * pause the ship momentarily
   */
  void pause();
  /**
   * set the aggressiveness level
   */
  virtual void setAggressiveness(int i);
  /**
   * notify if collided
   */
  void shipCollision();
  /**
   * get the current direction
   */
  int getCurrentDirection() { return _currentDirection; }

 protected:
  /**
   * run the ai code
   */
  virtual void runAI();

  /**
   * fire the beam to capture the
   *  single player
   */
  virtual void fireBeam();

  /**
   * timers
   */
  Timer _pauseTimer;
  Timer _invincibleTimer;
  Timer _collisionTimer;

  /**
   * draw the beam ?
   */
  bool _drawBeam;
  /**
   * texture for the beam
   */
  GLuint _beamTexture;

  /**
   * last distance from the player
   */
  double _lastDistance;
  /**
   * which direction are we moving
   */
  int _currentDirection;

  /**
   * aggressiveness level
   */
  int _aggressiveness;

  /**
   * degrees to spin the ship
   */
  double _spinDegrees;
};

#endif
